#ifndef _define_defines_
#define _define_defines_
#define ibm_pc_computer

#include <limits.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#define	TT	/**/

void error(char *fmt...);			// report about error

// definition of types

class bin;
class data;
class dataset;
class analog;

#define pbin	bin*
#define	plong	long*
#define pchar	char*
#define pdouble	double*

// CONSTANTS

// types of an activation function for p_funact

#define c_exp	0 /* logistic, range [0;1] */
#define c_th	1 /* th(x), range [-1;1] */
#define c_xx	2 /* x/(1+|x|) norm [-1;1] */
#define c_line	3 /* x - preferably for output layer only [1,-1] */
#define c_sigm	4 /* logistic function range [-0.5,0.5] */

//types of method analog complexation p_complex;
#define c_complex_one    0
#define c_complex_two    1
#define c_complex_three  2
#define c_comp_method    3

#define c_off		0	/* two standart switchs*/
#define c_on		1
#define c_two           2

#define c_datasets	10	/* maximal number of datasets to test*/
#define c_int_bit (CHAR_BIT*sizeof(int)/sizeof(char))
#define c_large_value	1e10 /* large value*/

// various
#define c_array		20000	/* maximal length of a string in input dataset*/

#define p_full			7		/* print option*/
#define p_float			4		/* print option : %p_full.p_float */
#define p_exp_lim		50.		/* value for cut-off of exponent in activation functions*/
#define c_eff_dif		20		/* if mse(c_s3) < mse(c_s1)*c_eff_dif == stop training*/

// maximal amount of layers
#define c_layers        3
#define c_max_layers   20

class param
{
long type;
long *a; double *b; char *s;
static float f;
	public:
long loaded,changed;
	param(){type=-1;s=NULL;loaded=changed=0;}
	void set(long *i, char* c){type=0; a=i; s=new char[strlen(c)+1];strcpy(s,c); loaded=changed=0;}
	void set(double *d, char* c){type=1; b=d; s=new char[strlen(c)+1];strcpy(s,c); loaded=changed=0;}
	char*name(){return s;}
	void copy(param p);
	long input(char *str,char *str1);
	void print(FILE *f,int &count,int mode);
	void compare(param p){changed+=type==1?*b!=*p.b:*a!=*p.a;}
};

// some external parameters

#define c_param_number 10 /* number of defined parameters*/


void init_param();

// dead variables (to use inside the programm)
extern int config_number;
extern time_t t_begin,t_current;
extern param global_param[c_param_number],save_param[c_param_number];
extern int p_layer_number, p_layers[c_layers],p_multiply,p_mask;
extern long p_seed_rand;
extern FILE *p_file[3],*p_out,*p_res_ou[3];
extern char file_name[100],environment[100];
extern double p_min_output,p_max_output,p_min_input,p_max_input,p_limit_init;

// input variables
extern long p_cut,p_show,p_print,p_name,p_norm,p_normv,
            p_rows,p_columns,p_n_output,p_method,p_grafic,p_optimum;

extern double p_max_correlation,p_limit;
extern bin* p_bmask;

double mse(double*x,double*y,long n);// calculate (x-y)^2 for n points
double calc_mse(double*x,double*y,long n);// calculate sqrt((x-y)^2/n) for n points
double ran1(long*);					// random value in (0,1)
long field_scanf(FILE*);				// return number of items in input string
FILE *mfopen(char *s,char *param);	// new fopen function
double mean_value(double *begin,double*end,long step); // calculate mean value of elements of an array
double correlation(double *b1,double*b2,double*end1,long step=1,long group=1);// calculate correlation amid elements of the arrays b1 & b2
double covariance(double *elem1,double *elem2,double*end,long step1,long step2); // ccalculate covariance amid two arrays
double dispersion(double *elem, double *end,double meanv,long step); // -//- dispersion of elements of an array
long max_abs_val(long,double*);	// return maximum absolute value in array
long min_abs_val(long,double*);	// return minimum absolute value in array
void copy_parameters(param destination[],param source[]);
void compare_parameters(param p1[],param p2[]);
extern "C" int compare_param(const void*,const void*);
extern "C" int compare_l(const void *a,const void *b);
extern "C" int compare_d(const void *a,const void *b);
void stifbs(double y[], double dydx[], long nv, double *xx, double htry, double eps,
	double yscal[], double *hdid, double *hnext,
	void (*derivs)(double, double [], double []));
void activate(double *value,long number,long funact);
double range_funact(long,long);
void print_param(FILE *f,long size,double *var);

inline void check_point(int point){if(point<0||point>2)error("in check_point=%i",point);}
inline void check_number(int n,int maxn){if(n<0||n>=maxn)error("in check_number=%i max=%i",n,maxn);}
inline double the_cachin_class(double a,double b){a-=b;return a<0.04&&a>-0.04;}
inline void memory_error(long n,char*s){error("Not enough memory for %li bytes in %s",n,s);}
inline double random_d(){return ran1(&p_seed_rand);}		// return random value in (0,1)
inline long random_i(long n){return long(n*random_d());}	// return long value in [0,n)
inline void set_random(long seed){p_seed_rand=seed<0?seed:seed?-seed:100;seed=(long)ran1(&p_seed_rand);}	// initialize random generator

class memory_array
{
long size,type;
union{double*d;long *i;}array;
	public:
	memory_array(long n,double*){type=size=0; if(n)memory_d(n);}
	memory_array(long n,long*){type=1;size=0; if(n)memory_i(n);}
	~memory_array(){if(size){if(type)delete array.i; else delete array.d;}}

	void local_error(long n){error("memory allocation error for %i bytes",n);}

	double* memory_d(long n){if(size<n){if(size)delete array.d;size=n;
							array.d=new double[size]; if(array.d==NULL)local_error(n*sizeof(double));
							}return array.d;}

	long* memory_i(long n){if(size<n){if(size)delete array.i;size=n;
							array.i=new long[size];if(array.i==NULL)local_error(n*sizeof(long));
							}return array.i;}
};

#endif

