!********************************************************************
! Select fittest models and save nModelSelect model objects to 
! aModelSelect array
!
! The procedure returns an error code of -1 if model was NOT selected,
! code 0 if model was selected and code > 0 if an error condition 
! occurred.
!
!	Arguments:
!		nSelModels - number models stored up to this iteration
!		modelLoc - location of selected model in array
!		maxIdx - array index of worst model
!		maxFVal - fitness value of worst model
!
!********************************************************************

	subroutine SelectModels( nSelModels, maxIdx, S, CR, IAn, SelAn )
									
	real(8)::	CR(LF), S
	integer::	IAn(ML11), SelAn(LF,ML11), nSelModels, maxIdx

      common /LSize/  ML, ML0, ML1, ML11, N, LF, MV,MV1,MV2, Qmax

	! if aModelSelect array not full
	if ( nSelModels < LF ) then 

		! Increment stored model counter
		nSelModels = nSelModels + 1	

		! Store new model at the end of aq array
		SelAn( nSelModels,: ) = IAn(:)
		CR( nSelModels ) = S

		! If array is full find worst model (max fitness value)
		! store its fitness value and its array index
		if ( nSelModels == LF ) then

			! Maximal CR value for saved models is set to zero 
			! because on first pass we want to find worst model
			maxIdx = maxloc( Cr, dim=1 )    ! Found position of global CRmax
		endif

	! If aModelSelect is full
	else
		! If current model better than worst one
		if ( S < CR( maxIdx )) then
			! Store current model instead of worst one
			SelAn( maxIdx,: ) = IAn(:)
			CR( maxIdx ) = S
			
			! Find model with maximal fitness value
			maxIdx = maxloc( Cr, dim=1 )    ! Found position of global CRmax			 
		endif 
	endif

	end subroutine SelectModels 
!********************************